FPPassFail <- function(RSfile, PDfile, CCLEfile, CCLEnamesFile, plateName){

## FPPassFail function

# For testing/debugging purposes
#  RSfile <- "./inputs/Plate_16_Replicate_screen_similarity.csv"
#  RSfile <- NULL
#  PDfile <- "./inputs/Plate_16_Parental_Cas9Deriv_similarity.csv"
#  PDfile <- "./inputs/PRISM_Plate_8_Parental_Cas9Deriv_similarity.csv"
#  CCLEfile <- "./inputs/Plate_16_SNP6toFluidigm_topresults.csv"
#  CCLEfile <- "./inputs/PRISM_Plate_8_SNP6toFluidigm_topresults.csv"
#  CCLEnamesFile <- "./inputs/CCLElineNames.csv"
#  plateName <- "Plate_16"
  
  
# CCLEfile should have all the cell lines and help us group them. Read this in first
CCLE <- read.csv(CCLEfile, colClasses = c("character","numeric", "numeric", "character", "character", "character", "character", "character"))
CCLE$KnownMatch.CCLE[is.na(CCLE$KnownMatch.CCLE)] <- "non"

# Separate into Parentals, Derivatives, and Screens
Parentals <- CCLE[CCLE$Reference.sample=="Y", ]
Derivatives <- CCLE[CCLE$Reference.Cas9.sample == "Y", ]
Screens <- CCLE[(CCLE$Reference.sample=="N" & CCLE$Reference.Cas9.sample == "N"), ]

PD <- read.csv(PDfile, colClasses = c("character","character","numeric", "numeric", "character", "character"))
PDlines <- lapply(PD$Input_line, function(x){strsplit(x,"__")[[1]][2]})
PD$Input_line <- PDlines
PD$FPmatch.itself[is.na(PD$FPmatch.itself)] <- "non"



RS <- read.csv(RSfile, colClasses = c("character","character","numeric", "numeric", "character", "character"))
RSlines <- lapply(RS$Input_line, function(x){strsplit(x,"__")[[1]][2]})
RS$Input_line <- RSlines
RS$FPmatch.itself[is.na(RS$FPmatch.itself)] <- "non"

# Read in CCLE sample info file
CCLElines <- read.csv(CCLEnamesFile)
CCLElines <- lapply(CCLElines, removeSpecialCharacters)

toc <- tic()
parentalLines <- unique(Parentals$FPCellLine)

results <- data.frame(line = character(length(parentalLines)), passfail = character(length(parentalLines)), reason = character(length(parentalLines)), stringsAsFactors = F)

derivativeLines <- unique(Derivatives$FPCellLine)


# Output a large file concatenating CCLE, PD, and RS files
nRows <- dim(CCLE)[1] + dim(PD)[1] + dim(RS)[1]
AllData <- data.frame(CellLine = character(length = nRows), n.SNPs = integer(length = nRows), 
                      identity = double(length = nRows), MatchedLine = character(length = nRows), 
                      FPmatch.CCLE = character(length = nRows), KnownMatch.CCLE = character(length = nRows), 
                      FPmatch.itself = character(length = nRows), PDS = character(length = nRows),
                      CCLEorM2M = character(length = nRows), matchIdentity = character(length = nRows), 
                      isLineInCCLE = character(length = nRows), PassFail = character(length = nRows), 
                      stringsAsFactors = F)
iter <- 1

for(i in 1:length(unique(CCLE$FPCellLine))){

  line <- unique(CCLE$FPCellLine)[i]
  line_clean <- clean_names(line)
  
  # Get information from this line
  lineCCLEinfo <- CCLE[which(CCLE$FPCellLine %in% line), ]
  
  if(lineCCLEinfo$Reference.sample == "N" & lineCCLEinfo$Reference.Cas9.sample == "N"){
    lineinfo <- RS[which(RS$Input_line %in% line), ]
    pds <- "S"
  } else {
    lineinfo <- PD[which(PD$Input_line %in% line), ]
    if(lineCCLEinfo$Reference.sample == "Y"){
      pds <- "P"
    } else {
      pds <- "D"
    }
  }
  
   # Could this ever be in CCLE?
    if(length(grep(line_clean,CCLElines)) != 0){
      CCLEexist <- "yes"
    } else {
      CCLEexist <- "no"
    }
  
  # Add to appropriate parts of the data frame
  
  for(j in 1:dim(lineCCLEinfo)[1]){
    #AllData$ArxSpan.IDs[iter] <- lineCCLEinfo$ArxSpan.IDs[j]
    AllData$CellLine[iter] <- lineCCLEinfo$FPCellLine[j]
    AllData$n.SNPs[iter] <- lineCCLEinfo$n.SNPs[j]
    AllData$identity[iter] <- lineCCLEinfo$identity[j]
    AllData$MatchedLine[iter] <- lineCCLEinfo$CleanCellLine.SNP6[j]
    AllData$FPmatch.CCLE[iter] <- lineCCLEinfo$FPmatch.CCLE[j]
    AllData$KnownMatch.CCLE[iter] <- lineCCLEinfo$KnownMatch.CCLE[j]
    AllData$PDS[iter] <- pds
    AllData$CCLEorM2M[iter] <- "ccle"
    
    mi <- matchIdentity(lineCCLEinfo$identity[j])
    AllData$matchIdentity[iter] <- mi
    AllData$isLineInCCLE[iter] <- CCLEexist
    
    
    if(AllData$PDS[iter] == "D"){
      
      AllData$PassFail[iter] <- derivative_ccle_passfail(AllData[iter, ])
      
    } else if(AllData$PDS[iter] == "P"){
      
      AllData$PassFail[iter] <- parental_ccle_passfail(AllData[iter, ])
      
    } else {
      
      AllData$PassFail[iter] <- screen_ccle_passfail(AllData[iter, ])
      
    }
    
    
    
    iter <- iter+1
    
  }
  if(dim(lineinfo)[1]>0){
    for(j in 1:dim(lineinfo)[1]){
      #AllData$ArxSpan.IDs[iter] <- lineinfo$ArxSpan.IDs[j]
      AllData$CellLine[iter] <- lineinfo$Input_line[j]
      AllData$n.SNPs[iter] <- lineinfo$n.SNPs[j]
      AllData$identity[iter] <- lineinfo$Similarity[j]
      AllData$MatchedLine[iter] <- lineinfo$Reference_line[j]
      AllData$FPmatch.itself[iter] <- lineinfo$FPmatch.itself[j]
      AllData$PDS[iter] <- pds
      AllData$CCLEorM2M[iter] <- "m2m"
      
      mi <- matchIdentity(lineinfo$Similarity[j])
      AllData$matchIdentity[iter] <- mi
      AllData$isLineInCCLE[iter] <- CCLEexist
      
      # Then we are testing a match2master row
      if(AllData$PDS[iter] == "D"){
        
        AllData$PassFail[iter] <- derivative_m2m_passfail(AllData[iter, ])
        
      } else if(AllData$PDS[iter] == "P"){
        
        AllData$PassFail[iter] <- parental_m2m_passfail(AllData[iter, ])
        
      } else {
        
        AllData$PassFail[iter] <- screen_m2m_passfail(AllData[iter, ])
        
      }
      
      
      
     
      
      iter <- iter+1
      
    }
  }  
    
  
  
}








AllData$CellLine <- as.character(AllData$CellLine)
AllData <- AllData[order(AllData$CellLine), ]
write.csv(AllData, paste("AllData_", plateName, ".csv", sep = ""), quote = F, row.names = F)



## Finally, use data in AllData to decide total passes.
iter2 <- 1
PassingLines <- character()
for(i in 1:length(unique(AllData$CellLine))){
  
  cellLine <- unique(AllData$CellLine)[i]
  
  passFails <- AllData$PassFail[which(AllData$CellLine %in% cellLine)]
  
  if(sum(passFails!="Pass")==0){
    
    PassingLines[iter2] <- cellLine
    iter2 <- iter2 + 1
  }
  
  
}

write.table(PassingLines, file = "PassingLines.txt", quote = F, row.names = F, col.names = F)

}

